'use strict';

angular.module('akio.webbox')

  /**
   * info pane directive
   */
  .directive('awbPaneWait', function() {

    function link(scope, iElement){
      // adjust sub-elements position & size
      var element = jQuery(iElement);

      element
        .find('.awb-pane-wait-body')
        .outerHeight('100%');
      element
        .find('.awb-pane-wait')
        .outerHeight(element.height());
    }

    return {
      restrict   : 'AE',
      replace    : true,
      templateUrl: 'app/layout/window/wait.tpl.html',
      link       : link
    };
  })

  /**
   * info pane controller
   * it handles
   */
  .controller('waitCtrl', ['$scope', '$timeout','$log','aip', 'navigation', 'state', 'session','notification', 'OpenMediaResultEventStatus',
    function( $scope, $timeout, $log, aip, navigation, state, session, notification, OpenMediaResultEventStatus){

      state.setCollapseIcon(true);

      var region             = state.getCenterRegion( ),
        mediumUuid           = region.mediumUuid,
        medium               = state.mediaList.getMediumByUuid( mediumUuid ),
        isRequestCancellable = true,
        events               = state.events( ).models,

        /**
         * create notification for each new view states when the box is collapsed
         */
        notifyViewStatesChange = function () {
          var addNotification = function(upStateText, overStateText, cancellable, handler) {
            notification.add(
              {
                kind: "media",
                states: [
                  { text: upStateText },
                  { text: overStateText }
                ]
              },
              handler,
              0,
              cancellable
            );
          };

          switch ($scope.info.viewState) {
            case 'WAITING':
              addNotification( $scope.labels.NOTIFICATION_OPENING_MEDIA_PENDING, $scope.labels.NOTIFICATION_CANCEL_CHAT_REQUEST, true, $scope.info.cancelOpenMediumRequest );
              break;

            case 'OPERATOR_JOINED_ROOM':
              addNotification( $scope.labels.NOTIFICATION_OPENING_MEDIA_PENDING, $scope.labels.NOTIFICATION_CANCEL_CHAT_REQUEST, true, $scope.info.cancelOpenMediumRequest );
              break;

            default: // manages the others states WHERE the plateform couldn't fulfill positively the demand
              addNotification(
                $scope.labels['NOTIFICATION_OPEN_CHAT_RESULT_' + $scope.info.viewState] || $scope.info.viewState + ' is not implemented yet',
                $scope.labels['NOTIFICATION_OPEN_CHAT_RESULT_' + $scope.info.viewState] || $scope.info.viewState + ' is not implemented yet',
                false
              );
          }
        },

        /**
         * resolves the current state after each new added event
         * @param event
         * @returns {boolean}
         */
        resolveViewState = function(event) {
          var result = false;
          state.setCollapseIcon(true);

          if (event.isOpenMediumResultEvent()) {
            $scope.info.viewState = event.get('status');

            // the wait panel is visible once a request to open a medium is pending.
            // It is hidden after the operator has joined the room...
            // The request is cancellable until the arrival of the event «OpenMediumResultEvent» meaning the medium is opened.

            if (event.get('status') !== OpenMediaResultEventStatus.OK) {
              $scope.titleBar.setPanelAsClosable(true);
              state.setTitle($scope.labels.WAITBOX_TITLE_INABILITY);
              state.events().off('add');
            }

            if ($scope.titleBar.isCollapsed()) {
              notifyViewStatesChange();
            }
            result = true;
          } else if (event.isOperatorJoinedEvent()) {
            // the room has been created...
            // ...But the agent's first message has not been received yet...
            // ...Still the user can't join the chat UI and send his message.
            isRequestCancellable = false;
            $scope.info.viewState = 'OPERATOR_JOINED_ROOM';
            result = true;
          }
          return result;
        },

        /**
         * WATCHES ui collapse state changes
         * the different stages of the request:
         *  - when the ui is expanded, are displayed on the wait screen
         *  - when the ui is collapsed, are notified to the user through notifications
         */
        unregisterUICollapseWatcher = $scope.$watch(
          function() {
            return $scope.titleBar.collapse;
          },
          function(newValue) {
            if (newValue === true) {
              notifyViewStatesChange( );
            } else {
              // remove notification
              notification.removeByKind( 'media' );
            }
          }
        );


      // this function is used to restart the box when it is in default waiting view (resolve toaster 240)
      $scope.restart = function() {
        if ($scope.info.viewState !== "WAITING" && $scope.info.viewState !== "OPERATOR_JOINED_ROOM" && isRequestCancellable === true) {
          isRequestCancellable = false;
          $timeout(function() {
            $scope.info.cancelOpenMediumRequest();
          }, 3000);
        }
      };

      //specific for testing purpose
      $scope.resolveViewState=function(event){resolveViewState(event);};
      $scope.notifyViewStatesChange=function(){notifyViewStatesChange();};
      //end of specific testing

      $scope.info = {


        viewState: OpenMediaResultEventStatus.WAITING,

        /**
         * holds the medium estimatedWaitTime
         */
        estimatedWaitTime: 0,

        /**
         * returns TRUE if the medium estimatedWaitTime attribute is defined and greater than 0
         * @returns {boolean}
         */
        isEstimatedWaitTime: function() {
          return !_.isNaN( $scope.info.estimatedWaitTime ) && $scope.info.estimatedWaitTime > 0;
        },

        /**
         * tells whether the request is CANCELLABLE at the moment
         * @returns {boolean}
         */
        canCancel: function() {
          return isRequestCancellable;
        },


        /**
         * CANCELS request to open medium
         */
        cancelOpenMediumRequest: function() {
          $log.debug('wait.cancelRequest');
          aip.closeMediaChat(mediumUuid)
            .then (function() {
              navigation.quit();
              state.setCurrentMediaType('');
            });
        },

        /**
         * ENTERS current medium
         */
        enterMedium: function() {
          navigation.enterMedia(mediumUuid);
        }

      };

      /**
       * garbage collector
       */
      $scope.$on('$destroy', function() {

        // clean notifications
        notification.removeByKind('media');

        // unregisters watchers
        unregisterUICollapseWatcher();
        // unregisterEventsWatcher();

        // unregisters listeners
        medium.off('change:estimatedWaitTime');
        state.events().off('add');
      });

      //-------
      // main
      //-------
      if (!_.isUndefined(medium)) {
        $scope.info.estimatedWaitTime = medium.get('estimatedWaitTime');
      }

      // updates window title
      state.setTitle($scope.labels.WAITBOX_TITLE);

      // assures current view state is correctly restored after page refresh
      if (!_.isEmpty(events)) {
        resolveViewState(_.last(events));
      }

      /**
       * listens to event list changes
       */
      state.events().on('add', function(model) {
        // maintains up to date the event list
        resolveViewState(model);
      });

      /**
       * listens to changes on medium 'estimatesWaitTime' property
       */
      medium.on('change:estimatedWaitTime', function(model, value) {
        $scope.info.estimatedWaitTime = value;
      });
    }
  ]);
