'use strict';

angular.module('akio.webbox')

  /**
   * the notification factory
   *
   * @usage
   * registering a non permanent and cancellable notification
   * notification.add(
   *  {
   *  kind: "chat",
   *  states: [
   *      {text: "up state"},
   *      {text: "over state"}
   *  ]},
   *  function(){console.log('callback: cancel the operation');}, 5,true);
   */

  .factory('notification', ['$timeout','$rootScope',
    function($timeout){
      var service = {
        list: {},

        /**
         * registers a 2 STATES (UP && OVER) notification.
         * @param item {object} notification description object. It should contains the declaration of one state minimum, 2 maximum.
         * @param callback {function | null} callback function triggered when the notification is clicked
         * @param delay {number} delay in *SECONDS*. a value of 0 makes the notification permanent.
         * @param cancellable {Boolean} a value of TRUE  and a callback declared makes the notification cancellable
         */
        add:function(item, callback, delay, cancellable){
          var timestamp = (new Date()).getTime();

          // the two states up & over are required
          // if only one is provided, it is duplicated for the 'over' state
          if (item.states.length === 1 /*&& item.kind!="error"*/){
            item.states.push({text:item.states[0].text});
          }

          var description = {
            kind: item.kind,
            details: item.details || [],
            states : item.states,
            isClickable: function () {
              return angular.isFunction(callback);
            },
            isError: function () {
              return item.kind === 'error';
            },
            isCancellable: function(){
              return angular.isFunction(callback) && cancellable === true;
            },
            click: function () {
              if (angular.isFunction(callback)) {
                delete service.list[timestamp];
                //searching for a link in the html
                var html = item.states[0].text;
                var urlArray = html.match(/href='([^']*')|href="([^"]*")/g);
                //if we found a link we open it in a new tab
                if(urlArray){
                  var urlToLoad = urlArray[0].slice(6,-1);
                  window.open(urlToLoad,'_blank');
                }
                callback();
              }
            }
          };

          if (angular.isDefined(delay) && Number(delay)>0){
            description.timer = $timeout(function () {
              delete service.list[timestamp];
            }, delay * 1000);
          }

          // one kind of notification can be displayed at the same time
          service.removeByKind(item.kind);

          service.list[timestamp] = description;
        },

        error: function(errorMessage, delay){
          if (!angular.isDefined(delay)){
            delay = 5;
          }

          this.add(
            {
              kind: "error",
              details: [],
              states: [
                {text: errorMessage}
              ]
            },
            null, delay
          );
        },

        /**
         * removes all items of a same kind
         * @param {String} kind the kind of notification we want to remove
         */
        removeByKind: function(kind){
          _.forIn(service.list, function(value, key) {
            if (kind === value.kind){
              if (value.timer){
                $timeout.cancel(value.timer);
              }
              delete service.list[key];
            }
          });
        }
      };

      return service;
    }
  ]);

