'use strict';

angular.module('akio.webbox')
  /**
   * local storage name
   */
  .constant('APP_STORAGE_NAME', 'awb-session')

  /**
   * session service
   */
  .provider('session', function () {
    var config = {
      id: null
    };

    return {
      setId: function (id) {
        config.id = id || config.id;
      },
      $get: ['state', 'trigger', 'media', '$log', '$injector', '$q', '$timeout', 'messageReader', 'ErrorCode', 'errorHandler', 'ViewState', 'MediumType', 'clickToChatButton', 'clickToWcbButton',
        function (state, trigger, media, $log, $injector, $q, $timeout, messageReader, ErrorCode, errorHandler, ViewState, MediumType, clickToChatButton, clickToWcbButton) {

          /**
           * delay between each polling
           * @type {number}
           */
          var boxParams = $injector.get('boxParams'),
            criticalError = false;

          /**
           * Get initialization data from AIP
           * @returns {*|deferred.promise|{then, always}|{then, catch, finally}}
           */
          var getInitDataFromAip = function () {
            return $injector.get('aip').getInitData();
          };

          /**
           * initiates navigation process
           */
          var startNavigation = function () {
            // state is sync and the view ready to be displayed
            $injector.get('navigation').start();

            // the app initializes successfully
            service.isInitialized = true;
          };


          //Web user inactivity starting
          var timeoutInMiliseconds = boxParams.WEB_USER_INACTIVITY_DELAY;
          var timeoutId;

          function startTimer() {
            timeoutId = window.setTimeout(doInactive, timeoutInMiliseconds);
          }

          function resetTimer() {
            window.clearTimeout(timeoutId);
            if (!service.isUserActive) {
              service.isUserActive = true;
              if (!service.fetchEventsIsRunning) {
                $timeout.cancel(service.fetchTimer);
                service.fetchMediaCounter = 0;
                service.fetchMedia(0);
              }
            }
            startTimer();
          }

          function doInactive() {
            service.isUserActive = false;
            service.fetchMediaCounter = 0;
          }

          function setupTimers() {
            document.addEventListener("mousemove", resetTimer, false);
            document.addEventListener("mousedown", resetTimer, false);
            document.addEventListener("keypress", resetTimer, false);
            document.addEventListener("touchmove", resetTimer, false);
            window.parent.window.document.addEventListener("mousemove", resetTimer, false);
            window.parent.window.document.addEventListener("mousedown", resetTimer, false);
            window.parent.window.document.addEventListener("keypress", resetTimer, false);
            window.parent.window.document.addEventListener("touchmove", resetTimer, false);
            startTimer();
          }

          setupTimers();
          //web user inactivity ending


          //onFocus/onBlur starting
          function onBlur() {
            if (service.fetchMediaIsRunning && !service.fetchEventsIsRunning) {
              service.fetchMediaIsRunning = false;
              service.webboxIsFocused = false;
              $timeout.cancel(service.fetchTimer);
            }
          }

          function onFocus() {
            if (!service.fetchMediaIsRunning && !service.fetchEventsIsRunning) {
              service.webboxIsFocused = true;
              service.fetchMediaCounter = 0;
              if (state.isTargetingRulesMatches()) {
                service.fetchMediaIsRunning = true;
                if (service.originalState !== null) {
                  service.fetchMedia(0);
                }
              }
            }
          }

          var isIE11 = !!window.MSInputMethodContext && !!document.documentMode;

          var addFocusBlurListener = function () {
            if (!isIE11) {
              window.parent.window.onfocus = onFocus;
              window.onfocus = onFocus;
              window.parent.window.onblur = onBlur;
              window.onblur = onBlur;
            } else {
              window.parent.window.onfocusin = onFocus;
              window.onfocusin = onFocus;
              window.parent.window.onfocusout = onBlur;
              window.onfocusout = onBlur;
            }
          };


          //onFocus/onBlur ending


          var service = { // return the service instance
            /**
             * initialization flag
             */
            isInitialized: false,

            isNetConnectionDown: false,

            fetchRetryCounter: 0,

            fetchMediaCounter: 0,

            isTryingToReconnect: false,

            fetchTimer: null,

            fetchMediaIsRunning: false,

            webboxIsFocused: true,

            fetchEventsIsRunning: false,

            calendarIsOk: false,
            /**
             * application state at start. Used to determine if a media is opened and has to be resumed.
             */
            originalState: null,

            getMediaRequestSent: false,

            isUserActive: true,

            //only for test purpose
            addFocusBlurListener: function () {
              addFocusBlurListener();
            },


            /**
             * used to modify polling delay in an exponential way
             **/
            addDelay: function (requestsNumber) {
              var delayToAdd = 1;
              var exponentialFactor = 2;
              for (var i = 0; i < requestsNumber - 1; i++) {
                delayToAdd = exponentialFactor * delayToAdd;
              }
              return delayToAdd / exponentialFactor;
            },

            /**
             * start session
             * @returns {*}
             */
            start: function () {
              var that = this,
                isActiveSession = config.id !== null,
                params = (isActiveSession) ? {id: config.id} : {},


                /**
                 * saves the config at start for further analysis
                 * @param result
                 * @returns {*}
                 */
                saveOriginalState = function (result) {
                  that.originalState = _.clone(result.get('regions'));
                  return result;
                },

                /**
                 * checks that the configuration with the server is set. If XHR2 is supported, the API connector is embedded in the application so we can suppose it is reachable.
                 */
                isAPIConnectorReady = function () {
                  return $injector.get('aip').ping();
                };

              $log.info(isActiveSession ? 'try to restore the active session...' : 'start over a session');

              return state.sync(params)
                .then(function (result) {
                  return result;
                })
                .then(saveOriginalState)
                .then(isAPIConnectorReady)
                .then(getInitDataFromAip)
                ["catch"](function (rejection) {
                $log.warn("------- session.start.failed", rejection);
                return $q.reject({text: rejection.data, errors: [rejection.errors || rejection.data]});
              })
                .then(addFocusBlurListener);
            },

            stopPolling: function () {
              service.fetchEventsIsRunning = false;
              service.fetchMediaIsRunning = false;
              console.log('polling stopped');
            },

            startPolling: function () {
              if (state.getDisplayAsButton()) {
                clickToChatButton.displayAsButton();
                clickToWcbButton.displayAsButton();
              }

              var polling;
              //we look at the events to know if a session is already start
              var events = state.events(),
                lastEvent = events.at(events.length - 1) || {},
                lastEventId = (lastEvent.attributes || {}).eventId || 0;
              //if there is events and no medium closed then a session exists and the webbox have to fetch new events
              if ((angular.isDefined(state.getChatSessionUri()) && state.getChatSessionUri() !== null && lastEventId === 0) || (lastEventId > 0 && !lastEvent.isMediumClosedEvent())) {
                service.fetchEventsIsRunning = true;
                polling = service.fetch;
                console.log('polling event start');
              } else { // else the webbox have to fetch media availability
                service.fetchMediaIsRunning = true;
                polling = service.fetchMedia;
                console.log('polling media start');

              }

              polling()
                .then(startNavigation)
                ["catch"](function (rejection) {
                $log.warn("------- session.start.failed", rejection);
                return $q.reject({text: rejection.data, errors: [rejection.errors || rejection.data]});
              });
            },

            /**
             * return state service
             * @returns {*}
             */
            getState: function () {
              return state;
            },


            /**
             * the BUSINESS RULE tells whether the box is authorized on a given page.
             * return TRUE if at least one media is available otherwise FALSE
             */
            businessRule: function () {
              return state.mediaList.hasSomeAvailable();
            },

            /**
             * the TARGET RULE tells if conditions are reunited to display the box
             */
            targetRule: function () {
              if (state.isLaunchBarMode()) {
                return state.get().get('targetingRules').display && state.getResultForAllCondition();
              } else {
                return state.get().get('targetingRules').display;
              }
            },

            /**
             * check if the box must be display as a button or as a launchBar. the state.get().get('displayAsButton') is defined in getInitData (AipConnector.js)
             *
             * @returns {Boolean}
             */

            isDisplayAsButton: function () {
              if (state.get()) {
                var isDisplay = state !== null ? state.getDisplayAsButton() : false;
                return isDisplay;
              }
            },

            isMediaChatOpenAndAvailable: function () {
              return state.mediaList.some(function(medium) {
                return !_.isUndefined(medium) && medium.get('uuid') === "2" && medium.isOpenedAndAgentAvailable();
              });
            },

            isMediaWcbOpenAndAvailable: function(){
              return state.mediaList.some(function(medium) {
                return !_.isUndefined(medium) && medium.get('uuid') === "8" && (medium.isOpenedAndAgentAvailable() || medium.isDelayedWebCallback());
              });
            },

            /**
             * checks whether the box is visible on the current page.
             * Two criteria must be validated:
             *  - is there an ongoing CHAT session that should be in priority followed up
             *  - is the box authorized on the page (business rule)
             * @returns {boolean}
             */
            isBoxVisible: function () {
              if (this.isInitialized) {
                return this.targetRule() && (this.businessRule() || this.isOngoingChatSession() || this.isOngoingWcbSession()) && !(state.isLaunchBarMode() && this.isNetConnectionDown);
              }
              return false;
            },

            /**
             * tells whether there is currently an ongoing chat session (that should be followed up in priority)
             * @returns {boolean}
             */
            isOngoingChatSession: function () {
              var centerRegion = state.getCenterRegion() || {} /* undefined if launchBar mode */,
                hasMedia = angular.isDefined((centerRegion || {}).mediumStatus),
                isChatMedia = centerRegion.mediumType === MediumType.MEDIUM_CHAT;
              return (hasMedia ? isChatMedia : false);
            },

            isOngoingWcbSession: function () {
              var centerRegion = state.getCenterRegion() || {} /* undefined if launchBar mode */,
                hasMedia = angular.isDefined((centerRegion || {}).mediumStatus),
                isWcbMedia = centerRegion.mediumType === MediumType.MEDIUM_WCB;

              return hasMedia ? isWcbMedia : false;
            },

            /**
             * tells whether there is an ongoing session meanings that the box is opened
             * @returns {boolean}
             */
            isBoxOpened: function () {
              return state.getCenterRegion(['mediumType']) !== ViewState.STATE_NONE;
            },

            /**
             * tells whether a trigger is currently active
             * @returns {boolean}
             */
            isActiveTrigger: function () {
              return trigger.exists();
            },

            /**
             * fetch mediaChat availability on the server
             * @param delay
             * @returns {*}
             */
            fetchMedia: function (delay) {
              if (!service.getMediaRequestSent) {
                if (state.getDisplayAsButton()) {
                  clickToChatButton.displayAppropriateButton(service.isMediaChatOpenAndAvailable(), service.isOngoingChatSession(), service.isNetConnectionDown);
                  clickToWcbButton.displayAppropriateButton(service.isMediaWcbOpenAndAvailable(), service.isOngoingWcbSession(), service.isNetConnectionDown);
                }
                var scheduleNextFetch = function (_delay) {
                    service.getMediaRequestSent = false;
                    if (service.fetchMediaIsRunning && service.webboxIsFocused) {
                      if (angular.isDefined(_delay)) {
                        delay = _delay;
                      }
                      if (!service.calendarIsOk) {
                        delay = boxParams.POLLING_DELAY_LOW;
                      }
                      if (service.fetchMediaCounter !== 0 && !service.isUserActive) {
                        delay = Math.min(boxParams.POLLING_DELAY_MEDIA + service.addDelay(service.fetchMediaCounter), boxParams.POLLING_DELAY_LOW);
                      } else {
                        delay = boxParams.POLLING_DELAY_MEDIA;
                      }
                      service.fetchTimer = $timeout(service.fetchMedia, delay);
                      return true;
                    }

                  },

                  fetchSuccess = function (result) {
                    service.isNetConnectionDown = false;
                    service.fetchMediaCounter++;

                    return messageReader
                      .handleResponse(result)
                      .then(scheduleNextFetch);
                  },

                  fetchError = function (rejection) {
                    service.isNetConnectionDown = true;
                    $log.error('---------fetchMedia failed...', rejection);
                    scheduleNextFetch(boxParams.HTTP_RETRY_ON_FAILURE_DELAY);
                    return $q.reject(rejection);
                  };
                service.getMediaRequestSent = true;

                return $injector.get('aip').getMedia().then(fetchSuccess, fetchError);
              }
            },

            /**
             * fetch data on the server
             * @returns {*}
             */
            fetch: function (delay) {

              if (state.getDisplayAsButton()) {
                clickToChatButton.displayAppropriateButton(service.isMediaChatOpenAndAvailable(), service.isOngoingChatSession());
                clickToWcbButton.displayAppropriateButton(service.isMediaWcbOpenAndAvailable(), service.isOngoingWcbSession());
              }

              var events = state.events(),
                lastEvent = events.at(events.length - 1) || {},
                lastEventId = (lastEvent.attributes || {}).eventId || 0,

                scheduleNextFetch = function (_delay) {
                  if (service.fetchEventsIsRunning && state.getChatSessionUri() !== null) {
                    if (angular.isDefined(_delay)) {
                      delay = _delay;
                    }
                    service.fetchTimer = $timeout(
                      function() {
                        if (service.fetchEventsIsRunning) {
                          service.fetch();
                        } else {
                          state.resetChatSessionUri();
                        }
                      }, delay);
                    return true;
                  }

                },

                fetchSuccess = function (result) {
                  service.isNetConnectionDown = false;
                  service.fetchRetryCounter = 0;
                  service.isTryingToReconnect = false;

                  //stop polling to fetch event if conversation closed
                  if (result.events) {
                    for (var i = 0; i < result.events.length; i++) {
                      if (result.events[i].type === 'MediumClosedEvent' /*&& result.events[i].sender.userType==='OPERATOR'*/) {
                        service.fetchEventsIsRunning = false;
                        service.fetchMediaIsRunning = true;
                        service.fetchMedia();
                      }
                    }
                  }


                  return messageReader
                    .handleResponse(result)
                    .then(scheduleNextFetch);
                },

                fetchError = function (rejection) {
                  $log.error('---------fetch failed...', rejection);
                  if (service.fetchRetryCounter < boxParams.ERROR_POLLING) {

                    if (state.isLaunchBarMode() || service.isNetConnectionDown /* do not notify the user again and again */) {
                      // the net connection is down the box will be automatically hidden
                      // we schedule the next fetch
                      if (service.isTryingToReconnect) {
                        scheduleNextFetch(boxParams.POLLING_DELAY_EVENT);
                        service.fetchRetryCounter++;
                      }

                    } else {
                      // we notify the user
                      errorHandler.interpret(rejection.code);

                      if (service.isTryingToReconnect) {
                        scheduleNextFetch(boxParams.POLLING_DELAY_EVENT);
                        service.fetchRetryCounter++;
                      }

                    }
                    service.isNetConnectionDown = true;
                  } else {
                    errorHandler.interpret(rejection.code);
                    service.isTryingToReconnect = false;
                    service.isNetConnectionDown = true;
                  }
                  return $q.reject(rejection);
                };
              delay = delay || boxParams.POLLING_DELAY_EVENT;

              return $injector.get('aip').fetch(lastEventId)
                .then(fetchSuccess, fetchError);
            },


            fetchWcbState: function (delay) {


              if (state.getDisplayAsButton()) {
                clickToWcbButton.displayAppropriateButton(service.isMediaWcbOpenAndAvailable(), service.isOngoingWcbSession());
              }


              var scheduleNextFetch = function (_delay) {
                  if (service.fetchEventsIsRunning) {
                    if (angular.isDefined(_delay)) {
                      delay = _delay;
                    }
                    service.fetchTimer = $timeout(service.fetchWcbState, delay);
                    return true;
                  }
                },

                fetchSuccess = function (result) {
                  service.isNetConnectionDown = false;
                  service.fetchRetryCounter = 0;
                  service.isTryingToReconnect = false;

                  //stop polling to fetch event if call picked up or failed
                  if (result === ('CALL_PICKED_UP_BY_CUSTOMER' || 'CALL_FAILED_LAST_ATTEMPT')) {
                    service.fetchEventsIsRunning = false;
                    service.fetchMediaIsRunning = true;
                    service.fetchMedia();
                  }

                  return state
                    .setWcbState(result)
                    .then(scheduleNextFetch);
                },

                fetchError = function (rejection) {
                  $log.error('---------fetch wcb state failed...', rejection);
                  if (service.fetchRetryCounter < boxParams.ERROR_POLLING) {

                    if (state.isLaunchBarMode() || service.isNetConnectionDown /* do not notify the user again and again */) {
                      // the net connection is down the box will be automatically hidden
                      // we schedule the next fetch
                      scheduleNextFetch(boxParams.POLLING_DELAY_EVENT);
                      service.fetchRetryCounter++;

                    } else {
                      // we notify the user
                      errorHandler.interpret(rejection.code);
                      scheduleNextFetch(boxParams.POLLING_DELAY_EVENT);
                      service.fetchRetryCounter++;
                      service.isNetConnectionDown = true;

                    }
                    service.isNetConnectionDown = true;
                  } else {
                    errorHandler.interpret(rejection.code);
                    service.isTryingToReconnect = false;
                    service.isNetConnectionDown = true;
                  }
                  return $q.reject(rejection);
                };
              delay = delay || boxParams.POLLING_DELAY_EVENT;

              return $injector.get('aip').fetchWcbState()
                .then(fetchSuccess, fetchError);
            },

            /**
             * try to catchup the interrupted session
             * @param delay  {Number|undefined}
             * @returns {*}
             */
            catchup: function (delay) { /* trigger events */
              return service.fetch(delay || 0);
            },

            /**
             * create a new session
             * @returns {*}
             */
            renew: function () { /* trigger events */
              var isOngoingChatSession = service.isOngoingChatSession(),
                centerRegion = state.getCenterRegion(),
                medium = {attributes: {type: centerRegion.mediumType, uuid: centerRegion.mediumUuid}} /* TRICKY */,
                navigation = $injector.get('navigation');

              state.dump();

              return service
                .start()
                .then(function () {
                  if (isOngoingChatSession) {
                    $injector.get('aip')
                      .openMediaChat()
                      .then(function () {
                        // request has been accepted
                        navigation.goto(medium);
                      });
                  }
                });
            },

            /**
             * call by the error handler in response to a critical error
             */
            quit: function () {
              $injector.get('navigation').quit();
              criticalError = true;
            },

            /**
             * stop session
             */
            stop: function () {
              $log.info('>>>>session is stopped');
              $timeout.cancel(this.fetchTimer);
            }
          };

          return service;
        }
      ]
    };
  })

  /**
   * executed during the provider registrations and configuration phase
   */
  .config(['APP_STORAGE_NAME', 'sessionProvider',
    function (APP_STORAGE_NAME, sessionProvider) {
      // try to resolve the session id of a potential ongoing session
      var sessionId = window.sessionStorage.getItem(APP_STORAGE_NAME); // return null otherwise
      sessionProvider.setId(sessionId);
    }
  ])

  .config(['$logProvider',
    function ($logProvider) {
      // activate log debug according to the value of the «awb-debug» session storage attribute
      if (window.sessionStorage) {
        var isDebug = window.sessionStorage.getItem('awb-debug') === 'true';
        $logProvider.debugEnabled(isDebug);
      }
    }
  ]);
